/*
    *
    * This file is a part of CorePDF.
    * A PDF viewer for C Suite.
    * Copyright 2019 CuboCore Group
    *

    *
    * This program is free software; you can redistribute it and/or modify
    * it under the terms of the GNU General Public License as published by
    * the Free Software Foundation; either version 3 of the License, or
    * (at your option) any later version.
    *

    *
    * This program is distributed in the hope that it will be useful,
    * but WITHOUT ANY WARRANTY; without even the implied warranty of
    * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    * GNU General Public License for more details.
    *

    *
    * You should have received a copy of the GNU General Public License
    * along with this program; if not, vsit http://www.gnu.org/licenses/.
    *
*/


#include "corepdf.h"
#include "ui_corepdf.h"
#include "qpdfpagenavigation.h"

#include <QtConcurrent/QtConcurrentRun>
#include <QApplication>
#include <QScroller>

#include <cprime/pinit.h>
#include <cprime/shareit.h>
#include <cprime/themefunc.h>
#include <cprime/appopenfunc.h>
#include <cprime/activitesmanage.h>
#include <cprime/cprime.h>


corepdf::corepdf(QWidget *parent) :QWidget(parent)
  , ui(new Ui::corepdf)
  , smi(new settings)
{
    ui->setupUi(this);

	// Set the transparent color to the main widget
	QPalette p(palette());
	p.setColor(QPalette::Base, Qt::transparent);
	this->setPalette(p);

    loadSettings();
    startsetup();
}

corepdf::~corepdf() {

    delete ui;
    delete PdfDoc;
    delete PdfWidget;
}


/**
// * @brief Setup ui elements
 */
void corepdf::startsetup() {

	ui->spinBox->setRange( 0, 0 );
	ui->spinBox->setPrefix( "Page " );
	ui->spinBox->setSuffix( " of 0" );

    // all toolbuttons icon size in sideView
    QList<QToolButton *> toolBtns = ui->sideView->findChildren<QToolButton *>();
    for (QToolButton *b: toolBtns) {
        if (b) {
            b->setIconSize(toolsIconSize);
        }
    }

    // all toolbuttons icon size in toolBar
    QList<QToolButton *> toolBtns2 = ui->toolBar->findChildren<QToolButton *>();
    for (QToolButton *b: toolBtns2) {
        if (b) {
            b->setIconSize(toolsIconSize);
        }
    }

    ui->page->setCurrentIndex(0);
    ui->sideView->setVisible( false );
    ui->menu->setEnabled(false);

    ui->appTitle->setAttribute(Qt::WA_TransparentForMouseEvents);
    ui->appTitle->setFocusPolicy(Qt::NoFocus);
    this->resize(800, 500);

    if (uiMode == 2) {
        this->setWindowState(Qt::WindowMaximized);

        ui->fileName->setVisible(0);
    }

    if(uiMode != 2 && activities){
        loadActivities();
    } else{
        ui->activitiesList->setVisible(false);
    }

    ui->page->setCurrentIndex(0);

    /* Page rotations: Hide the button if experimental features are not enabled */
    if ( not experimental )
        ui->rotate->hide();

    connect(ui->addFiles, SIGNAL(clicked()), this, SLOT(openFileDialog()));
    connect(ui->open, SIGNAL(clicked()), this, SLOT(openFileDialog()));

    PdfWidget = new QPdfView( this );
    PdfWidget->setPageMode( QPdfView::MultiPage );
    PdfWidget->setZoomMode( QPdfView::FitToWidth );
    ui->u->addWidget(PdfWidget);
    QScroller::grabGesture(PdfWidget, QScroller::LeftMouseButtonGesture);

    connect( PdfWidget->pageNavigation(), &QPdfPageNavigation::currentPageChanged, this, &corepdf::updateSpinBox );
};

/**
 * @brief Loads application settings
 */
void corepdf::loadSettings()
{
    activities = smi->getValue("CoreApps", "KeepActivities");
    listViewIconSize = smi->getValue("CoreApps", "ListViewIconSize");
    toolsIconSize = smi->getValue("CoreApps", "ToolsIconSize");
    uiMode = smi->getValue("CoreApps", "UIMode");
}

/**
 * @brief Sends file to library
 */
void corepdf::loadFile( const QString &file )
{
    workFilePath = file;
    ui->page->setCurrentIndex(1);

    PdfDoc = new QPdfDocument( file );

	QString text = "This document is password encrypted. Please enter the password to view it.";
	bool ok;
	while ( PdfDoc->passwordNeeded() ) {
		QString response = QInputDialog::getText(
			this,														// Parent
			"CorePDF | Enter Password",									// Dialog title
			text,														// Dialog text
			QLineEdit::Password,										// Echo mode
			QString(),													// Default text
			&ok															// Flag to check if the user cancels
		);

		/* User clicked cancel */
		if ( not ok ) {
			QMessageBox::warning( this, "CorePDF | Document Error", "Sorry! CorePDF cannot open this file without a password" );
            ui->page->setCurrentIndex(0);
			return;
		}

		/* Check if the password is correct */
		PdfDoc->setPassword( response );

		/* In case the password is wrong, an alternate text */
		text = "This password you entered is incorrect. Please try again.";
	}

    PdfDoc->load();

    PdfWidget->setDocument( PdfDoc );
    ui->fileName->setText( PdfDoc->title().isEmpty() ? PdfDoc->pdfName() : PdfDoc->title() );
    ui->menu->setEnabled( true );
    ui->rotate->setEnabled( true );

    ui->zoomCB->setCurrentIndex( smi->getValue( "CorePDF", "ZoomMode" ) );
    // PdfWidget->setZoomFactor( ( int )smi->getValue( "CorePDF", "ZoomFactor" ) * 1.0 / 100 );
    ui->pageModeCombo->setCurrentIndex( smi->getValue( "CorePDF", "PageMode" ) );

	ui->spinBox->setRange( 1, PdfDoc->pageCount() );
	ui->spinBox->setValue( 1 );
    ui->spinBox->setSuffix( QString( " of %1" ).arg( PdfDoc->pageCount() ) );
}

void corepdf::closeEvent( QCloseEvent *event )
{
    event->ignore();

	if (not workFilePath.isEmpty())
		CPrime::ActivitiesManage::saveToActivites("corepdf", QStringList() << workFilePath);

	PdfWidget->close();
    event->accept();
}

void corepdf::loadActivities()
{
    if(not activities){
        ui->activitiesList->setVisible(false);
        return;
    }

    ui->activitiesList->clear();

    QSettings recentActivity(CPrime::Variables::CC_ActivitiesFilePath(), QSettings::IniFormat);
    QStringList topLevel = recentActivity.childGroups();

    if (topLevel.count()) {
        topLevel = CPrime::SortFunc::sortDate(topLevel, CPrime::SortOrder::Descending);
    }

    QListWidgetItem *item;

    Q_FOREACH (QString group, topLevel) {
        recentActivity.beginGroup(group);
        QStringList keys = recentActivity.childKeys();
        keys = CPrime::SortFunc::sortTime(keys, CPrime::SortOrder::Descending, "hh.mm.ss.zzz");

        Q_FOREACH (QString key, keys) {
            QStringList value = recentActivity.value(key).toString().split("\t\t\t");
            if (value[0] == "corepdf") {
                QString filePath = value[1];
                if (not CPrime::FileUtils::exists(filePath))
                    continue;

                QString baseName = CPrime::FileUtils::baseName(filePath);
                item = new QListWidgetItem(baseName);
                item->setData(Qt::UserRole, filePath);
                item->setToolTip(filePath);
                item->setIcon(CPrime::ThemeFunc::getFileIcon(filePath));
                ui->activitiesList->addItem(item);
            }
        }

        recentActivity.endGroup();
    }
}

void corepdf::on_menu_clicked()
{
    if ( ui->sideView->isVisible() ) {
        ui->sideView->setVisible( false );
    } else {
        ui->sideView->setVisible( true );
    }
}

void corepdf::on_shareIt_clicked()
{
    if ( !workFilePath.isNull() ) {
        ShareIT *t = new ShareIT(QStringList() << workFilePath, listViewIconSize, nullptr);
        // Set ShareIT window size
        if (uiMode == 2 )
            t->setFixedSize(QGuiApplication::primaryScreen()->size() * .8 );
        else
            t->resize(500,600);

        t->exec();
    }
}

void corepdf::on_pinIt_clicked()
{
    if ( !workFilePath.isNull() ) {
        PinIT *pit = new PinIT(QStringList() << workFilePath, this);
        pit->exec();
    }
}

void corepdf::openFileDialog()
{
	QString fn = QFileDialog::getOpenFileName(
		this,
		"CorePDF - Open File",
		QDir::homePath(),
		"PDF Files (*.pdf);;All Files (*.*)"
	);

	if ( not fn.isEmpty() )
		loadFile( fn );
}

void corepdf::on_openmetadataApp_clicked() {

    CPrime::AppOpenFunc::defaultAppEngine( CPrime::DefaultAppCategory::MetadataViewer, workFilePath, QString() );
}

void corepdf::on_savePageAsPic_clicked() {

	int pageNo = PdfWidget->pageNavigation()->currentPage();
	QSizeF pSize = PdfDoc->pageSize( pageNo );

	/* Obtain the page zoom factor */
	QInputDialog *zoomDlg = new QInputDialog( this );
	zoomDlg->setInputMode( QInputDialog::IntInput );
	zoomDlg->setLabelText(
		QString(
			"Size of page %1 is %2 x %3 px. Please set the zoom factor below if necesssary."
		).arg( pageNo + 1 ).arg( pSize.width() ).arg( pSize.height() )
	);
	zoomDlg->setIntRange( 10, 400 );
	zoomDlg->setIntStep( 10 );
	zoomDlg->setIntValue( 100 );

	Q_FOREACH( QSpinBox *sb, zoomDlg->findChildren<QSpinBox *>() ) {
		if ( sb )
			sb->setSuffix( " %" );
	}

	if ( zoomDlg->exec() == QDialog::Accepted ) {

		/* Obtain the save file name */
		QStringList fmts;
		Q_FOREACH( QByteArray fmtba, QImageWriter::supportedImageFormats() ) {
			QString fmt = QString::fromLatin1( fmtba );
			fmts << fmt.toUpper() + " Image (*." + fmt + ")";
		}

		QString *str = new QString("PNG Image (*.png)");
		QString fn = QFileDialog::getSaveFileName(
			nullptr,
			"CorePDF - Save Current Page",
			QDir::homePath() + "/untitled.png",
			fmts.join( ";;" ) + ";;All Files (*.*)", str
		);

		if ( not fn.isEmpty() ) {
			PageSaver *saver = new PageSaver( PdfDoc->pdfPath() + "/" + PdfDoc->pdfName(), pageNo, zoomDlg->intValue(), fn );
			saver->start();
		}
	}
};

void corepdf::on_spinBox_valueChanged( int pageNo ) {

	if ( pageNo > 0 )
		if ( pageNo <= PdfDoc->pageCount() )
			PdfWidget->pageNavigation()->setCurrentPage( pageNo - 1 );

		else
			PdfWidget->pageNavigation()->setCurrentPage( PdfDoc->pageCount() - 1 );

	else
		PdfWidget->pageNavigation()->setCurrentPage( 0 );
};

void corepdf::on_zoomCB_currentIndexChanged( int idx ) {

	switch( idx ) {
		case 0: {
            PdfWidget->setZoomMode( QPdfView::FitToWidth );
			break;
		}

		case 1: {
            PdfWidget->setZoomMode( QPdfView::FitInView );
			break;
		}

		default: {
			PdfWidget->setZoomMode( QPdfView::CustomZoom );
			qreal zoom = ui->zoomCB->currentText().replace( "%", "" ).toInt() * 1.0 / 100;
//            sm->setValue( "CorePDF", "ZoomMode", ui->zoomCB->currentIndex() );
			PdfWidget->setZoomFactor( zoom );
		}
	}
}

void corepdf::on_rotate_clicked() {

	qDebug() << "Rotating page" << PdfWidget->pageNavigation()->currentPage();
	PdfDoc->rotatePage( PdfWidget->pageNavigation()->currentPage() );
}

void corepdf::on_pageModeCombo_currentIndexChanged( int idx ) {

	if ( not PdfDoc or not PdfWidget )
		return;

	switch( idx ) {
		case 0: {
            PdfWidget->setPageMode( QPdfView::PageMode::SinglePage );
			break;
		}

		case 1: {
            PdfWidget->setPageMode( QPdfView::PageMode::MultiPage );
			break;
		}
	}
}

void corepdf::updateSpinBox( int pageNo )
{
	ui->spinBox->setValue( pageNo + 1 );
}

PageSaver::PageSaver( QString name, int page, int zoomVal, QString saveName ) {
	pdfName = name;
	pageNo = page;
	zoom = zoomVal;
	imgName = saveName;
};

void PageSaver::run() {

	Poppler::Document *mPdfDoc = Poppler::Document::load( pdfName );

//	if ( not mPdfDoc ) {
//        CPrime::MessageEngine::messageEngine(
//            useSystemNotification,
//			"CorePDF",
//			"corepdf",
//			"Failed to save page as image",
//            "Don't know what's the cause, open an issue to let the devs know."
//		);
//		qDebug() << "Poppler::Document load failed" << pdfName;
//		return;
//	}

//    if ( mPdfDoc->isLocked() ) {
//        CPrime::MessageEngine::messageEngine(
//            useSystemNotification,
//			"CorePDF",
//			"corepdf",
//			"Failed to save page as image",
//            "Don't know what's the cause, open an issue to let the devs know."
//		);
//		qDebug() << "Poppler::Document is locked";
//		return;
//	}

	mPdfDoc->setRenderHint( Poppler::Document::Antialiasing );
	mPdfDoc->setRenderHint( Poppler::Document::TextAntialiasing );
	mPdfDoc->setRenderHint( Poppler::Document::TextHinting );

	qDebug() << pdfName << pageNo << zoom << imgName;

	PdfPage *p = mPdfDoc->page( pageNo );
	QImage image = p->renderToImage( 72.0 * zoom / 100.0, 72.0 * zoom / 100.0, -1, -1, -1, -1 );

	if ( not image.isNull() ) {
		bool ret = image.save( imgName );
		qDebug() << "Saved file" << ( ret ? "[SUCCESS]" : "[FAILED]" );
		qDebug() << "Image" << image.size();

//		if ( ret ) {
//			CPrime::InfoFunc::messageEngine(
//				"CorePDF",
//				"corepdf",
//				"Image saved successfully",
//				"Page successfully saved as image.",
//				nullptr
//			);
//		}

//		else {
//			CPrime::InfoFunc::messageEngine(
//				"CorePDF",
//				"corepdf",
//				"Failed to save page as image",
//				"Don't know what's the cause, open an issue to let the devs know.",
//				nullptr
//			);
//		}
	}

	delete mPdfDoc;
	delete p;
};

void corepdf::on_activitiesList_itemDoubleClicked(QListWidgetItem *item)
{
    QString filepath = item->data(Qt::UserRole).toString();
    loadFile( filepath );
}
